/*
 * test source file for assembling to ELF
 * build with:
 *    nasm -f elf elftest.asm
 *    gcc -o elftest elftest.c elftest.o
 * (assuming your gcc is ELF)
 */


#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <math.h>

extern double DNRM2(int, double *, int);
double DNRM2_C(int n, double *x, int incx);
void test();
void testRandom();

int main(int argc, char **argv) {
	if (argc == 1) {
		printf("Test mode\n");
		test();
		printf("Test mode completed.\n");
		return 0;
	} else {
		int components = argc - 1;
		double *values = (double *) malloc (components * sizeof(double));
		int i;
		for (i = 0; i < components; i++) {
			values[i] = atof(argv[i + 1]);
		}
		printf("Result: %lf\n", DNRM2(components, values, sizeof(double)));
		free(values);
	}	
}

void test() {
	double values [] = {5, 4, 3, 6};
	assert (DNRM2(0, values, 8) == 0);
	assert (DNRM2(-1, values, 8) == -1);
	assert (DNRM2(5, 0, 8) == -1);
	assert (DNRM2(5, values, -1) == -1);
	double largevalues [] = { 2e999, 2e99 };
	assert (DNRM2(2, largevalues, 8) == 1.0/0.0);
	int i;
	for (i = 0; i < 5000; i++) {
		testRandom();
	}
}

void testRandom() {
	int length = rand() % 50;
	int i;
	double *array = (double *) malloc (length * sizeof(double));
	for (i = 0; i < length; i++) {
		array[i] = rand();
	}
	printf("%lf\n", DNRM2(length, array, sizeof(double)));
	printf("%lf\n", DNRM2_C(length, array, sizeof(double)));
	// Int-Vergleich da Rundungsfehler auftreten
	// FPU hat bis zu 96-Bit zur Verfgung aber der C-Code speichert den Wert durch die Zwischenschritte wieder als 64 bit
	// dadurch geht Genauigkeit verloren
	assert (((int) DNRM2(length, array, sizeof(double))) == ((int) DNRM2_C(length, array, sizeof(double))));
	free(array);
}

double DNRM2_C(int n, double *x, int incx) {
	if (x == NULL || n < 0 || incx <= 0) {
		return -1;
	}
	if (n == 0) {
		return 0;
	}

	char *ptr = (char *) x;
	double sum = 0;
	double value;
	int i = 0;
	for (i = 0; i < n; i++) {
		value = *((double *) ptr);
		sum += value * value;
		ptr += incx;
	}
	return sqrt(sum);
}
